
body.modal {
    overflow: hidden;
}

.dialog-background {
    z-index: 10000;
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background-color: #666;
    opacity: 0.85;
    filter: alpha(opacity=85);
}

.Dialog {
    position: fixed;
    z-index: 10001;
    background-color: white;
    box-shadow: 2px 2px 3px #555;
    border-radius: 0.5em;
    padding: 0.5em;
    padding-top: 0;
    border: 1px solid #666;
}

.Dialog .close_dialog_button,
.Dialog .close_dialog_button:hover {
    position: absolute;
    right: 0.5em;
    top: 0.5em;
    border: none;
    padding: 0;
    margin: 0;
    cursor: pointer;
    background-color: transparent;
    background-image: none;
}

.Dialog .dialog_header {
    padding: 1.4em 0;
}

.Dialog .dialog_heading {
    margin: 0;
    color: #333;
    font-family: Arial;
    font-size: 1.5em;
}

