#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Website
#------------------------------------------------------------------------------
translations.define("Website.notices",
    ca = u"Avisos",
    es = u"Avisos",
    en = u"Notices"
)

# NoticesBox
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.notices.NoticesBox.close_notice_button",
    ca = u"Ocultar aquest missatge",
    es = u"Ocultar este mensaje",
    en = u"Hide this message"
)

