#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Configuration
#------------------------------------------------------------------------------
translations.define("Configuration.media.audio",
    ca = u"Audio",
    es = u"Audio",
    en = u"Audio"
)

translations.define("Configuration.audio_decoders",
    ca = u"Decodificadors d'àudio",
    es = u"Decodificadores de audio",
    en = u"Audio decoders"
)

translations.define("Configuration.audio_encoders",
    ca = u"Codificadors d'àudio",
    es = u"Codificadores de audio",
    en = u"Audio encoders"
)

# AudioDecoder
#------------------------------------------------------------------------------
translations.define("AudioDecoder",
    ca = u"Decodificador d'àudio",
    es = u"Decodificador de audio",
    en = u"Audio decoder"
)

translations.define("AudioDecoder-plural",
    ca = u"Decodificadors d'àudio",
    es = u"Decodificadores de audio",
    en = u"Audio decoders"
)

translations.define("AudioDecoder.mime_type",
    ca = u"Tipus MIME",
    es = u"Tipo MIME",
    en = u"MIME type"
)

translations.define("AudioDecoder.command",
    ca = u"Comanda",
    es = u"Comando",
    en = u"Command"
)

# AudioEncoder
#------------------------------------------------------------------------------
translations.define("AudioEncoder",
    ca = u"Codificador d'àudio",
    es = u"Codificador de audio",
    en = u"Audio encoder"
)

translations.define("AudioEncoder-plural",
    ca = u"Codificadors d'àudio",
    es = u"Codificadores de audio",
    en = u"Audio encoders"
)

translations.define("AudioEncoder.identifier",
    ca = u"Identificador",
    es = u"Identificador",
    en = u"Identifier"
)

translations.define("AudioEncoder.mime_type",
    ca = u"Tipus MIME",
    es = u"Tipo MIME",
    en = u"MIME type"
)

translations.define("AudioEncoder.extension",
    ca = u"Extensió",
    es = u"Extensión",
    en = u"Extension"
)

translations.define("AudioEncoder.command",
    ca = u"Comanda",
    es = u"Comando",
    en = u"Command"
)

translations.define("AudioEncoder.timeout",
    ca = u"Temps màxim d'execució",
    es = u"Tiempo máximo de ejecución",
    en = u"Timeout"
)

translations.define("AudioEncoder.timeout_size_factor",
    ca = u"Temps d'execució addicional segons la mida del fitxer",
    es = u"Tiempo de ejecución adicional según la mida del fichero",
    en = u"Timeout file size factor"
)

