
.SurveyResultsView .item_body {
    padding: 1em 0.7em;
}

.SurveyResultsView .answers_heading {
    font-size: 1.4em;
    margin-top: 2em;
    margin-bottom: 0.5em;
}

.SurveyResultsView .SurveyQuestionResultsView .question {
    font-size: 1em;
    margin-top: 1.5em;
    margin-bottom: 0.5em;
}

.SurveyResultsView .SurveyOptionsQuestionResultsView .label_cell {
    width: 20em;
}

.SurveyResultsView .SurveyOptionsQuestionResultsView .value_cell {
    text-align: right;
    width: 8em;
    padding-right: 1em;
}

.SurveyResultsView .SurveyOptionsQuestionResultsView .bar_cell {
    width: 20em;
}

.SurveyResultsView .SurveyOptionsQuestionResultsView .option_bar {
    background-color: #516ca7;
    border-radius: 0.2em;
    height: 1.2em;
}

